SystemOrganization addCategory: #'SMock-Core'!
SystemOrganization addCategory: #'SMock-Tests'!

TestCase subclass: #ArgumentSpecTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SMock-Tests'!

!ArgumentSpecTest methodsFor: 'test cases' stamp: 'dk 17.8.07 11:58'!
testAnyArgs
	"Verify that the #withAnyArgs selector works correctly."

	| spec |
	spec := ArgumentSpec new withAnyArgs.

	self should: [ spec matches: #() ].
	self should: [ spec matches: #(1) ].
	self should: [ spec matches: #($a 2 '3') ].
	self shouldnt: [ spec matches: nil ].! !

!ArgumentSpecTest methodsFor: 'test cases' stamp: 'dk 17.8.07 11:58'!
testDoesNotMatchNil
	"Verify that an ArgumentSpec does not match a nil argument list"
	| spec |
	spec := ArgumentSpec new add: 1.

	self shouldnt: [ spec matches: nil ]! !

!ArgumentSpecTest methodsFor: 'test cases' stamp: 'dk 17.8.07 11:58'!
testDoesNotMatchNonBlockObject
	"Verify that an ArgumentSpec correctly does not match for simple (and non-matching) object arguments."
	| spec |
	spec := ArgumentSpec new add: 1.

	self shouldnt: [ spec matches: #(nil) ]! !

!ArgumentSpecTest methodsFor: 'test cases' stamp: 'dk 17.8.07 11:58'!
testDoesNotMatchTooFewArguments
	"Verify that an ArgumentSpec does not match a argument list with too few arguments"
	| spec |
	spec := ArgumentSpec new add: 1.

	self shouldnt: [ spec matches: #() ]! !

!ArgumentSpecTest methodsFor: 'test cases' stamp: 'dk 17.8.07 11:58'!
testDoesNotMatchTooManyArguments
	"Verify that an ArgumentSpec does not match a argument list with too many arguments"
	| spec |
	spec := ArgumentSpec new add: nil.

	self shouldnt: [ spec matches: #(nil nil) ]! !

!ArgumentSpecTest methodsFor: 'test cases-promiscuity' stamp: 'dk 17.8.07 11:58'!
testIsPromiscuous1
	"Verify that a newly created ArgumentSpec is promiscuous."

	self should: [ ArgumentSpec new isPromiscuous ]! !

!ArgumentSpecTest methodsFor: 'test cases-promiscuity' stamp: 'dk 17.8.07 11:58'!
testIsPromiscuous2
	"Verify that a newly created ArgumentSpec which has had #withAnyArgs has been called is promiscuous."

	self should: [ ArgumentSpec new withAnyArgs isPromiscuous ]! !

!ArgumentSpecTest methodsFor: 'test cases' stamp: 'dk 17.8.07 11:58'!
testMatchesNonBlockObject
	"Verify that an ArgumentSpec correctly does not match for simple (and non-matching) object arguments."
	| spec |
	spec := ArgumentSpec new add: 1.

	self should: [ spec matches: #(1) ]! !

!ArgumentSpecTest methodsFor: 'test cases' stamp: 'dk 17.8.07 11:58'!
testMatchingBlock
	"Verify that a block is correctly evaluated as non-matching."
	| spec block |
	block := [ :ignored | true ].
	spec := ArgumentSpec new add: block.

	self should: [ spec matches: #(nil) ]! !

!ArgumentSpecTest methodsFor: 'test cases' stamp: 'dk 17.8.07 11:58'!
testMultiMatch
	| spec |
	spec := ArgumentSpec new addAll: (OrderedCollection new add: 1; add: [ :ignored | true ]; yourself).

	self should: [ spec matches: #(1 0) ]! !

!ArgumentSpecTest methodsFor: 'test cases' stamp: 'dk 17.8.07 11:58'!
testMultiNonMatch
	| spec |
	spec := ArgumentSpec new addAll: (OrderedCollection new add: 1; add: [ :ignored | false ]; yourself).

	self shouldnt: [ spec matches: #(1 0) ]! !

!ArgumentSpecTest methodsFor: 'test cases' stamp: 'dk 17.8.07 11:58'!
testNonMatchingBlock
	"Verify that a block is correctly evaluated as non-matching."
	| spec block |
	block := [ :ignored | false ].
	spec := ArgumentSpec new add: block.

	self deny: ( spec matches: #(block) )! !

!ArgumentSpecTest methodsFor: 'test cases-promiscuity' stamp: 'dk 17.8.07 11:58'!
testNotPromiscuous1
	"Verify that an ArgumentSpec with parameter restrictions defined is not prmiscuous."

	| argSpec |

	argSpec := ArgumentSpec new.

	argSpec add: 6.

	self shouldnt: [ argSpec isPromiscuous ]! !

!ArgumentSpecTest methodsFor: 'test cases-promiscuity' stamp: 'dk 17.8.07 11:58'!
testNotPromiscuous2
	"Verify that #withAnyArgs makes and ArgumentSpec promiscuous"

	| argSpec |

	argSpec := ArgumentSpec new.

	argSpec add: 6.

	self shouldnt: [ argSpec isPromiscuous ].

	argSpec withAnyArgs.

	self should: [ argSpec isPromiscuous ]! !

TestCase subclass: #BlockOrValueTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SMock-Tests'!

!BlockOrValueTest methodsFor: 'test cases' stamp: 'dk 17.8.07 11:58'!
testBlockValue
	"Verify that a BlockOrValue with a Block value returns the value of executing the Block."
	| blockOrValue |
	blockOrValue := BlockOrValue new value: [ 6 = 6 ].

	self should: [ blockOrValue value = true ]! !

!BlockOrValueTest methodsFor: 'test cases' stamp: 'dk 17.8.07 11:58'!
testNonBlockValue
	"Verify that a BlockOrValue with a non-Block value returns the value."
	| blockOrValue |
	blockOrValue := BlockOrValue new value: 6.

	self should: [ blockOrValue value = 6 ]! !

!BlockOrValueTest methodsFor: 'test cases' stamp: 'dk 17.8.07 11:58'!
testUninitialized
	"Verify that a newly created BlockOrValue returns nil."
	| blockOrValue |
	blockOrValue := BlockOrValue new.

	self should: [ blockOrValue value = nil ]! !

TestCase subclass: #MethodStubTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SMock-Tests'!

!MethodStubTest methodsFor: 'test cases-action' stamp: 'dk 17.8.07 11:58'!
testActionSpecified
	"Verify that #runAction runs the specified action"
	| stub x |
	x := 10.
	stub := (MethodStub newForSelector: #abc) will: [ x := 20 ].
	stub runAction.
	self assert: x = 20! !

!MethodStubTest methodsFor: 'test cases-arguments' stamp: 'dk 17.8.07 11:58'!
testAnyArgs
	"Verify that the #withAnyArgs selector works correctly."

	| unaryStub keyword1Stub |
	unaryStub := (MethodStub newForSelector: #size) withAnyArgs.
	keyword1Stub := (MethodStub newForSelector: #add:) withAnyArgs.

	self assert: (unaryStub matches: (Message selector: #size arguments: #())).
	self assert: (unaryStub matches: (Message selector: #size arguments: #(1))).
	
	self assert: (keyword1Stub matches: (Message selector: #add: arguments: #())).
	self assert: (keyword1Stub matches: (Message selector: #add: arguments: #(1))).
	self assert: (keyword1Stub matches: (Message selector: #add: arguments: #(1 2))).! !

!MethodStubTest methodsFor: 'test cases-arguments' stamp: 'dk 17.8.07 11:58'!
testMatchesBlockArgs
	"Verify that object args match correctly"
	| stub |
	stub := (MethodStub newForSelector: #whatever) with: [ :arg | arg > 6 ]; with: [ :arg | arg < 5 ].

	self deny: (stub matches: (Message selector: #whatever arguments: #(6 5))).
	self deny: (stub matches: (Message selector: #whatever arguments: #())).
	self deny: (stub matches: (Message selector: #whatever arguments: #(5 6))).
	self deny: (stub matches: (Message selector: #whatever arguments: #(6 7))).
	self assert: (stub matches: (Message selector: #whatever arguments: #(7 4)))! !

!MethodStubTest methodsFor: 'test cases-arguments' stamp: 'dk 17.8.07 11:58'!
testMatchesNoArgs
	"Verify that a simple selector match works"

	| stub message |
	stub := MethodStub newForSelector: #printString.
	message := Message selector: #printString.

	self assert: (stub matches: message)! !

!MethodStubTest methodsFor: 'test cases-arguments' stamp: 'dk 17.8.07 11:58'!
testMatchesObjectArgs
	"Verify that object args match correctly"
	| stub |
	stub := (MethodStub newForSelector: #whatever) with: 6; with: 5.

	self should: [ stub matches: (Message selector: #whatever arguments: #(6 5)) ].
	self shouldnt: [ stub matches: (Message selector: #whatever arguments: #()) ].
	self shouldnt: [ stub matches: (Message selector: #whatever arguments: #(5 6)) ].
	self shouldnt: [ stub matches: (Message selector: #whatever arguments: #(6 7)) ]! !

!MethodStubTest methodsFor: 'test cases-arguments' stamp: 'dk 17.8.07 11:58'!
testMatchesWithArgsButNoStubbedArgs
	"Verify that there is a match if the stub has no args specified but the Message does"

	| stub message |
	stub := MethodStub newForSelector: #printString.
	message := Message selector: #printString arguments: #(1 'a' $b).

	self should: [ stub matches: message ]! !

!MethodStubTest methodsFor: 'test cases-creation' stamp: 'dk 17.8.07 11:58'!
testNewMethodStub
	"Test the initial state of a newly created MethodStub."

	| stub |

	stub := MethodStub newForSelector: #abc.

	self shouldnt: [ stub wasInvoked ]! !

!MethodStubTest methodsFor: 'test cases-action' stamp: 'dk 17.8.07 11:58'!
testNoAction
	"Verify that #runAction doesn't barf when no action has been specified"
	| stub |
	stub := MethodStub newForSelector: #abc.
	stub runAction! !

!MethodStubTest methodsFor: 'test cases-return' stamp: 'dk 17.8.07 11:58'!
testNoReturnSpecified
	"Verify that a MethodStub with no return specified returns the alternate."
	| stub |
	stub := MethodStub newForSelector: #whatever.
	self should: [ (stub returnValueOr: 6) = 6 ]! !

!MethodStubTest methodsFor: 'test cases-return' stamp: 'dk 17.8.07 11:58'!
testReturnsNil
	"Verify that a MethodStub with a nil specified as the return value returns the nil."
	| stub |
	stub := (MethodStub newForSelector: #whatever) returns: nil.
	self should: [ (stub returnValueOr: 6) isNil ]! !

!MethodStubTest methodsFor: 'test cases-return' stamp: 'dk 17.8.07 11:58'!
testReturnsObject
	"Verify that a MethodStub with an object specified as the return value returns the object."
	| stub |
	stub := (MethodStub newForSelector: #whatever) returns: $a.
	self should: [ (stub returnValueOr: 6) = $a ]! !

!MethodStubTest methodsFor: 'test cases-action' stamp: 'dk 17.8.07 11:58'!
testWasInvokedWithAction
	"Verify that the MethodStub tracks invocation when an action is specified."

	| stub |

	stub := MethodStub newForSelector: #abc.
	stub will: [ "do something" ].
	stub runAction.

	self assert: stub wasInvoked.! !

!MethodStubTest methodsFor: 'test cases-action' stamp: 'dk 17.8.07 11:58'!
testWasInvokedWithNoAction
	"Verify that the MethodStub tracks invocation even if no action is specified."

	| stub |

	stub := MethodStub newForSelector: #abc.
	stub runAction.

	self assert: stub wasInvoked.! !

TestCase subclass: #MockTest
	instanceVariableNames: 'mock proxy'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SMock-Tests'!

!MockTest methodsFor: 'setup/teardown' stamp: 'dk 17.8.07 11:58'!
setUp
	mock := Mock new.
	proxy := mock proxy.! !

!MockTest methodsFor: 'test cases-expectations' stamp: 'dk 17.8.07 11:58'!
testCallingExpectationMultipleTimes
	"There is a bug where, if you define an expectation once and then call it multiple times, a SubScriptOutOfBoundsError is signalled in Mock>>findMethodStubMatching:in:allowingInvoked:"

	mock expect: #doit.

	self shouldnt: [ proxy doit ] raise: MockingException.
	self should: [ proxy doit ] raise: MockingException! !

!MockTest methodsFor: 'test cases-stubs' stamp: 'dk 17.8.07 11:58'!
testCallingStubMultipleTimes
	"Verify that there are no problems calling a stub multiple times even if it has been defined only once."

	mock stub: #doit.

	self shouldnt: [ proxy doit ] raise: MockingException.
	self shouldnt: [ proxy doit ] raise: MockingException.
	self shouldnt: [ proxy doit ] raise: MockingException.! !

!MockTest methodsFor: 'test cases-proxy' stamp: 'dk 17.8.07 11:58'!
testCallingUnstubbedMethod
	"Test the result of sending a message to a Proxy when the proxied object natively does understand the message but there has been no method stub set up for it.
	Expected result: MockingException should be thrown."

	self should: [ proxy add: 1 ] raise: MockingException.! !

!MockTest methodsFor: 'test cases-stubs' stamp: 'dk 17.8.07 11:58'!
testClosestMatch
	"Verify that, when there is more than one MethodStub matching the method call, the more exact match is used."
	| x |

	(mock stub: #add:) withAnyArgs; will: [ x := #Bad ].
	(mock stub: #add:) with: 6; will: [ x := #Good ].

	mock proxy add: 6.

	self should: [ x = #Good ]! !

!MockTest methodsFor: 'test cases-exceptions' stamp: 'dk 17.8.07 11:58'!
testDefaultException
	"Test the result of using a Mock that uses the default Exception.
	Expected result: MockingException will be signaled when a stub is made for a method not understood by the mocked class."

	mock := Integer mock.

	self should: [ mock stub: #add: ] raise: MockingException.
	self should: [ mock expect: #add: ] raise: MockingException.

	[ mock stub: #add: ] on: MockingException do: [ :ex | 'Integer does not understand #add:.' = ex messageText ].! !

!MockTest methodsFor: 'test cases-stubs' stamp: 'dk 17.8.07 11:58'!
testDefaultReturn
	"Test the result of calling a stubbed method that does not specify a return.
	Expected result: The proxy itself will be returned"

	mock stub: #abs.

	self should: [ proxy abs == proxy ]! !

!MockTest methodsFor: 'test cases-expectations' stamp: 'dk 17.8.07 11:58'!
testExpectingRandomMethod
      "Verify that a Mock created with no class specified can understand any message."

	self shouldnt: [ mock expect: #/ ] raise: MockingException.
	self shouldnt: [ mock expect: #something: ] raise: MockingException.
	self shouldnt: [ mock expect: #thisUnaryDoesNotExist ] raise: MockingException.! !

!MockTest methodsFor: 'test cases-expectations' stamp: 'dk 8/17/2007 12:41'!
testExpectingUnknownMethod
	"Verify that expecting a method that the mocked class doesn't understand results in a MockingException."

	mock := String mock.
	
	self should: [ mock expect: #bla ] raise: MockingException.
	[ mock expect: #bla ] on: MockingException do: [ :ex | self assert: ex messageText = 'String does not understand #bla.' ].! !

!MockTest methodsFor: 'test cases - #findMethodStubMatching:in:allowingInvoked:' stamp: 'dk 17.8.07 11:58'!
testFindMethodStubMatchingWithoutAllowingInvoked
	| message stubs methodStub1 methodStub2 actual |

	message := Message selector: #add: argument: 1.
	methodStub1 := (MethodStub newForSelector: #add:) with: 1.
	methodStub2 := (MethodStub newForSelector: #add:) with: 1.
	methodStub1 runAction.

	stubs := OrderedCollection new add: methodStub1; add: methodStub2; yourself.

	actual := mock findMethodStubMatching: message in: stubs allowingSatisfied: false.

	self should: [ methodStub2 = actual ]! !

!MockTest methodsFor: 'test cases-proxy' stamp: 'dk 17.8.07 11:58'!
testMultipleSelectorMatch
	"Test the result of calling a method where the selector has been stubbed more than once but the arguments match only on one stub."
	| x |
	x := nil.
	(mock stub: #add:) with: [ :arg | arg > 10 ]; will: [ x := 10 ].
	(mock stub: #add:) with: [ :arg | arg < 10 ]; will: [ x := 20 ].

	proxy add: 100.
	self should: [ x = 10 ].
	
	proxy add: 1.
	self should: [ x = 20 ]! !

!MockTest methodsFor: 'test cases-proxy' stamp: 'dk 17.8.07 11:58'!
testProxyClassMultiple
	"Test that a Mock returns the same instance of Proxy every time that #proxy is called"

	self should: [ proxy == mock proxy ]! !

!MockTest methodsFor: 'test cases-stubs' stamp: 'dk 17.8.07 11:58'!
testReturnNil
	"Test the result of calling a stubbed method that specifies a return value  of nil.
	Expected result: nil"

	(mock stub: #abs) returns: nil.

	self shouldnt: [ proxy abs == proxy ].
	self should: [ proxy abs isNil ]! !

!MockTest methodsFor: 'test cases-exceptions' stamp: 'dk 17.8.07 11:58'!
testSpecifiedException
	"Test the result of using a Mock that uses a specified Exception
	Expected result: TestFailure will be signaled when a stub is made for a method not understood by the mocked class."

	mock := Mock newForClass: Integer withException: TestFailure.

	self should: [ mock stub: #add: ] raise: TestFailure.
	self should: [ mock expect: #add: ] raise: TestFailure.

	[ mock stub: #add: ] on: TestFailure do: [ :ex | 'Integer does not understand #add:.' = ex messageText ].! !

!MockTest methodsFor: 'test cases-stubs' stamp: 'dk 17.8.07 11:58'!
testStubWithExecute
	"Test the result of calling a stubbed method that specifies a body but no return.
	Expected result: The body will be executed, and the return value will be the Proxy"

	| x |

	x := nil.
	(mock stub: #abs) will: [ x := 'data'].

	self should: [ proxy abs = proxy ].
	self should: [ x = 'data' ]! !

!MockTest methodsFor: 'test cases-stubs' stamp: 'dk 17.8.07 11:58'!
testStubWithExecuteAndReturn
	"Test the result of calling a stubbed method that specifies a return.
	Expected result: The reutrn value"

	| x |

	x := 'a'.
	(mock stub: #abs) will: [ x := 99 ]; returns: 'test'.

	self should: [ proxy abs = 'test' ].
	self should: [ x = 99 ]! !

!MockTest methodsFor: 'test cases-stubs' stamp: 'dk 17.8.07 11:58'!
testStubWithReturn
	"Test the result of calling a stubbed method that specifies a return.
	Expected result: The reutrn value"

	(mock stub:  #abs) returns: 'test'.

	self should: [ proxy abs = 'test' ]! !

!MockTest methodsFor: 'test cases-stubs' stamp: 'dk 17.8.07 11:58'!
testStubbingRandomMethod
      "Verify that a Mock created with no class specified can understand any message."

	self shouldnt: [ mock stub: #/ ] raise: MockingException.
	self shouldnt: [ mock stub: #something: ] raise: MockingException.
	self shouldnt: [ mock stub: #thisUnaryDoesNotExist ] raise: MockingException.! !

!MockTest methodsFor: 'test cases-stubs' stamp: 'dk 17.8.07 11:58'!
testStubbingUnknownMethod
	"Verify that stubbing a method that the mocked class doesn't understand results in a MockingException."

	mock := Integer mock.

	self should: [ mock stub: #add: ] raise: MockingException.
	[ mock stub: #add: ] on: MockingException do: [ :ex | self assert: ex messageText = 'Integer does not understand #add:.' ].! !

!MockTest methodsFor: 'test cases-verification' stamp: 'dk 17.8.07 11:58'!
testVerifyWithMetExpectations
	"Test that calling #verify doesn't raise VerificationException for a Mock that has defined expectations that have been met."

	(mock expect: #add:) withAnyArgs.

	mock proxy add: 5.

	self shouldnt: [ mock verify ] raise: VerificationException.! !

!MockTest methodsFor: 'test cases-verification' stamp: 'dk 17.8.07 11:58'!
testVerifyWithNoStubsOrExpectations
	"Test the effects of calling verify on a Mock that doesn't have any expectations or stubs set.
	Expected result: the verification will pass (VerificationException not thrown)"

	self shouldnt: [ mock verify ] raise: VerificationException.! !

!MockTest methodsFor: 'test cases-verification' stamp: 'dk 17.8.07 11:58'!
testVerifyWithOnlyStubsSet
	"Test calling #verify on a Mock which only has stubs.  Expected result: VerificationException is not raised."

	(mock stub: #add:) withAnyArgs.
	mock stub: #size.

	self shouldnt: [ mock verify ] raise: VerificationException.! !

!MockTest methodsFor: 'test cases-verification' stamp: 'dk 17.8.07 11:58'!
testVerifyWithUnmetExpectations
	"Test that calling #verify does raise VerificationException for a Mock that has defined expectations that have not been met."

	(mock expect: #add:) withAnyArgs.

	self should: [ mock verify ] raise: VerificationException.! !

!MockTest methodsFor: 'test cases-verification' stamp: 'dk 17.8.07 11:58'!
testVerifyingSameMethodStubTwice
	"Verify that if a MethodStub is expected twice (same selector and args) and called twice, that the Mockv verifies correctly"

	| vals |
	vals := #(1 2 1 3 1 4).

	vals
		do: [ :val | (mock expect: #add:) with: val ].

	vals
		do: [ :val | proxy add: val ].

	self shouldnt: [ mock verify ] raise: VerificationException! !

TestCase subclass: #ReturnValueSpecTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SMock-Tests'!

!ReturnValueSpecTest methodsFor: 'test cases' stamp: 'dk 17.8.07 11:58'!
testBlockReturnValueSet
	"Verify that the ReturnValueSpec's specified block return value is evaluated and its result is returned returned instead of the alternate return value.  The block return value must be a 0-argument block."
	| spec |
	spec := ReturnValueSpec new returns: [ 1 = 2 ].

	self should: [ (spec returnValueOr: 6) = false ]! !

!ReturnValueSpecTest methodsFor: 'test cases' stamp: 'dk 17.8.07 11:58'!
testNoReturnValueSetAndAlternateIsBlock
	"Verify that the alternate return value is returned when there is no return value.  When the return value is a block, then the block is evaluated and it's resulting value is returned.  The block must be a 0-argument block."
	| spec |
	spec := ReturnValueSpec new.

	self should: [ (spec returnValueOr: [ 1 = 1 ]) = true ]! !

!ReturnValueSpecTest methodsFor: 'test cases' stamp: 'dk 17.8.07 11:58'!
testNoReturnValueSetAndAlternateIsObject
	"Verify that the alternate return value is returned when there is no return value.  When the alternate is a non-block object, then the alternate itself is returned."
	| spec |
	spec := ReturnValueSpec new.

	self should: [ (spec returnValueOr: 6) = 6 ]! !

!ReturnValueSpecTest methodsFor: 'test cases' stamp: 'dk 17.8.07 11:58'!
testNonBlockReturnValueSet
	"Verify that the ReturnValueSpec's specified non-block return value is returned instead of the alternate return value."
	| spec |
	spec := ReturnValueSpec new returns: $a.

	self should: [ (spec returnValueOr: 6) = $a ]! !

TestCase subclass: #SMBlockClosureTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SMock-Tests'!

!SMBlockClosureTest methodsFor: 'test cases' stamp: 'dik 5/23/2010 00:08'!
testIsBlockClosure

	self should: [ [] isKindOf: BlockClosure]! !

TestCase subclass: #SMClassTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SMock-Tests'!

!SMClassTest methodsFor: 'test cases' stamp: 'dk 17.8.07 11:58'!
testClassMockIsMock
	"Verify that the Mock returned by Class #mock is the same as Mock forClass: aClass."

	| mock1 mock2 |
	mock1 := String mock.
	mock2 := Mock newForClass: String.

	self should: [ mock1 mockedClass = mock2 mockedClass ]! !

TestCase subclass: #SMObjectTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SMock-Tests'!

!SMObjectTest methodsFor: 'test cases' stamp: 'dk 17.8.07 11:58'!
testIsBlockContext
	"Verify that #isBlockContext returns false for an Object."

	self shouldnt: [ Object new isBlockContext ]! !

TestCase subclass: #SMProxyTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SMock-Tests'!

!SMProxyTest methodsFor: 'test cases-defaults' stamp: 'dk 8/17/2007 12:53'!
testDefaultDebugString

	| proxy |
	proxy := SMProxy newForMock: ( Mock newForClass: Integer ).

	self shouldnt: [ proxy debugString ] raise: MessageNotUnderstood.
	self should: [ proxy debugString = 'a SMProxy for Integer' ]! !

!SMProxyTest methodsFor: 'test cases-defaults' stamp: 'dk 17.8.07 11:58'!
testDefaultEquals
	"Verify that, if an equals is not specified, the default equals runs"

	| proxy |
	proxy := SMProxy newForMock: Integer mock.

	self shouldnt: [ proxy = proxy ] raise: MessageNotUnderstood.
	self shouldnt: [ proxy = 'dummy' ].
	self should: [ proxy = proxy ]! !

!SMProxyTest methodsFor: 'test cases-exceptions' stamp: 'dk 17.8.07 11:58'!
testDefaultException
	"Verify that MockingException will be signaled when a method is called on a Proxy and the method hasn't been stubbed in the Mock."

	| proxy |
	proxy := (Mock newForClass: OrderedCollection) proxy.

	self should: [ proxy add: 'whatever' ] raise: MockingException.

	[ proxy add: 'whatever' ] on: MockingException do: [ :ex | 'There is no stub for class OrderedCollection matching: #add: with arguments: #(''whatever'').' ]! !

!SMProxyTest methodsFor: 'test cases-defaults' stamp: 'dk 8/17/2007 12:53'!
testDefaultInspectString

	| proxy |
	proxy := SMProxy newForMock: OrderedCollection mock.

	self shouldnt: [ proxy inspectString ] raise: MessageNotUnderstood.
	self should: [ proxy inspectString = 'a SMProxy for OrderedCollection' ]! !

!SMProxyTest methodsFor: 'test cases-defaults' stamp: 'dk 8/17/2007 12:46'!
testDefaultPrintString

	| proxy |
	proxy := SMProxy newForMock: Object mock.

	self shouldnt: [ proxy printString ] raise: MessageNotUnderstood.
	self should: [ proxy printString = 'a SMProxy for Object' ]! !

!SMProxyTest methodsFor: 'test cases-stubbed methods' stamp: 'dk 17.8.07 11:58'!
testMethodNotProxiedWithClass
	"Test the result of sending a message to a Proxy when the specified message has not been stubbed..
	Expected result: MockingException should be thrown."

	| proxy |
	proxy := Integer mock proxy.
	self should: [ proxy add: 1 ] raise: MockingException.
	[ proxy add: 1 ] on: MockingException do: [ :ex | self assert: ex messageText = 'There is no stub for class Integer matching: #add: with arguments: #(1).' ]! !

!SMProxyTest methodsFor: 'test cases-stubbed methods' stamp: 'dk 17.8.07 11:58'!
testMethodNotProxiedWithoutClass
	"Test the result of sending a message to a Proxy when the specified message has not been stubbed..
	Expected result: MockingException should be thrown."

	| proxy |
	proxy := Mock new proxy.
	self should: [ proxy add: 1 ] raise: MockingException.
	[ proxy add: 1 ] on: MockingException do: [ :ex | self assert: ex messageText = 'There is no stub matching: #add: with arguments: #(1).' ]! !

!SMProxyTest methodsFor: 'test cases-stubbed methods' stamp: 'dk 17.8.07 11:58'!
testNoArgumentRestrictionsSpecified
	"Verify that calling a proxied method that hasn't had any argument restrictions specified is the same as calling a proxied method that has had withAnyArgs specified."

	| mock |
	mock := OrderedCollection mock.
	(mock stub: #add:).

	self shouldnt: [ mock proxy add: 1 ] raise: MockingException.! !

!SMProxyTest methodsFor: 'test cases-exceptions' stamp: 'dk 17.8.07 11:58'!
testSpecifiedException
	"Verify that TestFailure will be signaled when a method is called on a Proxy and the method hasn't been stubbed in the Mock."

	| proxy |
	proxy := (Mock newForClass: OrderedCollection withException: TestFailure ) proxy.

	self should: [ proxy add: 'whatever' ] raise: TestFailure.

	[ proxy add: 'whatever' ] on: TestFailure do: [ :ex | 'There is no stub for class OrderedCollection matching: #add: with arguments: #(''whatever'').' ]! !

!Class methodsFor: '*SMock-Core-mocking' stamp: 'dk 17.8.07 11:58'!
mock
	^ Mock newForClass: self! !

ProtoObject subclass: #SMProxy
	instanceVariableNames: 'mock'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SMock-Core'!

SMProxy class
	instanceVariableNames: 'DefaultStubs'!

SMProxy class
	instanceVariableNames: 'DefaultStubs'!

!SMProxy class methodsFor: 'class initialization' stamp: 'dk 17.8.07 11:57'!
initialize
	"Initialize class variables"
	| printStringBlock printOnBlock |
	printStringBlock := [ :rcvr |
			| aStream |
			aStream := WriteStream on: (String new: 16).
			rcvr printOn: aStream.
			aStream contents ].

	printOnBlock := [ :rcvr :aStream |
			| title |
			title := rcvr class printString.
			rcvr xxxproxiedClass ifNotNil: [ title := title , ' for ' , rcvr xxxproxiedClass printString ].
			aStream nextPutAll: ((title at: 1) isVowel ifTrue: ['an '] ifFalse: ['a ']).
			aStream nextPutAll: title ].

	DefaultStubs := Dictionary new
		at: #class put: [ :rcvr | rcvr xxxtype ];
		at: #hash put: [ :rcvr | rcvr identityHash ];
		at: #= put: [ :rcvr :arg | rcvr == arg ];
		at: #inspectString put: printStringBlock;
		at: #debugString put: printStringBlock;
		at: #printString put: printStringBlock;
		at: #displayString put: printStringBlock;
		at: #isBlockContext put: [ :rcvr | false ];

		"The two below are added for inspecting"
		at: #inspect put: [ :rcvr | Inspector openOn: rcvr ];
		at: #basicPrintString put: printStringBlock;

		"Added so that Trippy doesn't get upset"
		at: #inspectorActions put: [ :rcvr | #() ];

		"For debugging"
		at: #isKindOf: put: [ :rcvr :aClass | self class includesBehavior: aClass ];
		at: #printOn: put: printOnBlock;
		yourself! !

!SMProxy class methodsFor: 'instance creation' stamp: 'dk 17.8.07 11:57'!
newForMock: aMock
	^ super new xxxinitializeForMock: aMock! !

!SMProxy methodsFor: 'testing' stamp: 'dk 8/17/2007 17:18'!
= anObject
	^self == anObject
	! !

!SMProxy methodsFor: 'local-accessing' stamp: 'dik 5/27/2012 14:58'!
class
	"Primitive. Answer the object which is the receiver's class. Essential. See 
	Object documentation whatIsAPrimitive."

	<primitive: 111>
	self primitiveFailed! !

!SMProxy methodsFor: 'printing' stamp: 'dk 8/17/2007 12:53'!
debugString
	^self printString! !

!SMProxy methodsFor: 'forwarding' stamp: 'dk 8/17/2007 12:55'!
doesNotUnderstand: aMessage
	| stub |

	"Find a stub matching the method call, raise exception if it doesn't exist"
	stub := mock findMethodStubMatching: aMessage.

	stub ifNil: [
					| message |
					message := 'There is no stub', (mock mockedClass ifNil: [ '' ] ifNotNil: [ ' for class ', mock mockedClass printString]), ' matching: ' , aMessage selector printString , ' with arguments: ' , aMessage arguments printString, '.'.
					mock exceptionClass signal: message ] .

	"run the stub's body (if specified)"
	stub runAction.

	"return the stub's return value (if specified)"
	^ stub returnValueOr: self! !

!SMProxy methodsFor: 'testing' stamp: 'dk 8/17/2007 17:18'!
hash
	^self identityHash 
	! !

!SMProxy methodsFor: 'printing' stamp: 'dk 8/17/2007 12:53'!
inspectString
	^self printString! !

!SMProxy methodsFor: 'local-accessing' stamp: 'dk 8/17/2007 17:22'!
inspectorClass
	"Answer the class of the inspector to be used on the receiver.  Called by inspect; 
	use basicInspect to get a normal (less useful) type of inspector."

	^ Inspector! !

!SMProxy methodsFor: 'local-accessing' stamp: 'dk 8/17/2007 17:13'!
instVarAt: index
	"Primitive. Answer a fixed variable in an object. The numbering of the 
	variables corresponds to the named instance variables. Fail if the index 
	is not an Integer or is not the index of a fixed variable. Essential. See 
	Object documentation whatIsAPrimitive."

	<primitive: 73>
	
	nil primitiveFailed ! !

!SMProxy methodsFor: 'local-accessing' stamp: 'dk 8/17/2007 17:13'!
instVarAt: index put: anObject
	"Primitive. Answer a fixed variable in an object. The numbering of the 
	variables corresponds to the named instance variables. Fail if the index 
	is not an Integer or is not the index of a fixed variable. Essential. See 
	Object documentation whatIsAPrimitive."

	<primitive: 74>
	
	nil primitiveFailed ! !

!SMProxy methodsFor: 'testing' stamp: 'dk 8/17/2007 17:15'!
isBlockContext
	^false! !

!SMProxy methodsFor: 'testing' stamp: 'dk 8/17/2007 17:16'!
isLiteral
	^false! !

!SMProxy methodsFor: 'testing' stamp: 'dk 8/17/2007 17:17'!
isSelfEvaluating
	^self isLiteral! !

!SMProxy methodsFor: 'testing' stamp: 'dk 8/17/2007 17:15'!
notNil
	^true! !

!SMProxy methodsFor: 'printing' stamp: 'LaurentLaffont 12/27/2010 12:27'!
printOn: aStream
	| title |
	title := self class name.
	aStream
		nextPutAll: (title first isVowel ifTrue: ['an '] ifFalse: ['a ']);
		nextPutAll: title.
	aStream nextPutAll: ' for '.
	self xxxproxiedClass printOn: aStream! !

!SMProxy methodsFor: 'printing' stamp: 'dk 8/17/2007 17:23'!
printString
	"Answer a String whose characters are a description of the receiver. 
	If you want to print without a character limit, use fullPrintString."

	^ self printStringLimitedTo: 50000! !

!SMProxy methodsFor: 'printing' stamp: 'LaurentLaffont 12/27/2010 12:26'!
printStringLimitedTo: limit
	"Answer a String whose characters are a description of the receiver.
	If you want to print without a character limit, use fullPrintString."
	| limitedString |
	limitedString := String streamContents: [:s | self printOn: s] limitedTo: limit.
	limitedString size < limit ifTrue: [^ limitedString].
	^ limitedString , '...etc...'! !

!SMProxy methodsFor: 'private' stamp: 'dik 9/3/2007 17:57'!
xxxclass
	"Answer the object which is the receiver's class."

	<primitive: 111>
	nil primitiveFailed! !

!SMProxy methodsFor: 'initialize-release' stamp: 'dk 17.8.07 11:57'!
xxxinitializeForMock: aMock
	mock := aMock.! !

!SMProxy methodsFor: 'local-testing' stamp: 'dk 17.8.07 11:57'!
xxxisProxy
	^ true! !

!SMProxy methodsFor: 'local-accessing' stamp: 'dk 17.8.07 11:57'!
xxxproxiedClass
	^ mock mockedClass! !

!SMProxy methodsFor: 'initialize-release' stamp: 'dk 17.8.07 11:57'!
xxxrelease
	mock := nil! !

!SMProxy methodsFor: 'local-accessing' stamp: 'dk 17.8.07 11:57'!
xxxtype
	"Answer the object which is the receiver's class."

	<primitive: 111>
	nil primitiveFailed! !

Object subclass: #ArgumentSpec
	instanceVariableNames: 'argList'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SMock-Core'!

!ArgumentSpec commentStamp: '<historical>' prior: 0!
An ArgumentSpec is the argument specification for a MethodStub.  It records the expectations/limitations put on specific method arguments, and then verifies that the arguments used in the call to a Mock's SMProxy matches the expectations/limitations.

Instance Variables:
	isPromiscuous	<Boolean>	Whether or not this ArgumentSpec matches any/all arguments.
	argList	<Collection>	Collection of argument limitations/expectations

!

!ArgumentSpec class methodsFor: 'instance creation' stamp: 'dk 17.8.07 11:57'!
new
	^ super new initialize! !

!ArgumentSpec methodsFor: 'adding' stamp: 'dk 17.8.07 11:57'!
add: anArgument
	"Add an argument to this ArgumentSpec.  The argument may be any object -- be aware that BlockContexts are handled differently than other objects."

	argList add: anArgument.! !

!ArgumentSpec methodsFor: 'adding' stamp: 'dk 17.8.07 11:57'!
addAll: aCollection
	"Add all of the arguments from aCollection to this ArgumentSpec.  The arguments may be any object -- be aware that BlockContexts are handled differently than other objects."

	argList addAll: aCollection! !

!ArgumentSpec methodsFor: 'initialize-release' stamp: 'dk 17.8.07 11:57'!
initialize
	argList := OrderedCollection new.! !

!ArgumentSpec methodsFor: 'querying' stamp: 'dk 17.8.07 11:57'!
isPromiscuous
	"Return whether this ArgumentSpec is promiscuous -- ie/ matches any/all arguments."

	^ argList isEmpty! !

!ArgumentSpec methodsFor: 'querying' stamp: 'dk 17.8.07 11:57'!
matches: aCollection
	"Test whether this ArgumentSpec matches the arguments in aCollection."

	| match |

	aCollection ifNil: [ ^ false ].
	self isPromiscuous ifTrue: [ ^ true ].
	aCollection size ~= argList size ifTrue: [ ^ false ].

	match := true.
	argList with: aCollection do: [ :definedSpec :candidate |
		match := match and: [ self spec: definedSpec matches: candidate ] ].

	^ match! !

!ArgumentSpec methodsFor: 'printing' stamp: 'dk 17.8.07 11:57'!
printOn: aStream
	aStream nextPutAll: '#('.

	1 to: argList size do: [ :idx |
		(argList at: idx) printOn: aStream.
		idx == argList size
			ifFalse: [ aStream nextPut: Character space ] ].

	aStream nextPut: $)! !

!ArgumentSpec methodsFor: 'querying-private' stamp: 'dik 5/23/2010 00:17'!
spec: argSpec matches: anArgument
	^ argSpec class = BlockClosure 
		ifTrue: [ argSpec value: anArgument ]
		ifFalse: [ argSpec = anArgument ]! !

!ArgumentSpec methodsFor: 'adding' stamp: 'dk 8/17/2007 12:54'!
withAnyArgs
	"Indicate that this ArgumentSpec matches any arguments."

	argList reset! !

Object subclass: #BlockOrValue
	instanceVariableNames: 'value'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SMock-Core'!

!BlockOrValue commentStamp: '<historical>' prior: 0!
A BlockOrValue is a wrapper for Objects which does special handling of BlockContexts.  The wrapped Object is set with the #value: method, and returned with the #value method.  If the wrapped object is a BlockContext, #value will return the result of executing the BlockContext.  Otherwise, #value returns the object itself.

Instance Variables:
	value	<Object>	The wrapped object

!

!BlockOrValue methodsFor: 'value' stamp: 'dik 5/23/2010 00:12'!
value
	"Return this BlockOrValue's value unless the value is an executable Block, in which case evaluate the block and return its result."

	^ (value class = BlockClosure)
			ifTrue:[value value]
			ifFalse: [value]! !

!BlockOrValue methodsFor: 'value' stamp: 'dk 17.8.07 11:57'!
value: anObject
	"Set this BlockOrValue's value.  May be a Block."

	value := anObject! !

Object subclass: #MethodStub
	instanceVariableNames: 'argSpec action returnValueSpec selector invocationCount'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SMock-Core'!

!MethodStub commentStamp: '<historical>' prior: 0!
MethodStub is used by the Mock framework to specify method stubs.  A method stub defines which methods it responds to and how.  MethodStubs are permissive in that any attribute that isn't specified matches everything.  For example, if there aren't any argument limitations added to a MethodStub, then the MethodStub will match any collection of arguments.  Essentially, the user puts limitations on what a MethodStub is permitted to match.

For a MethodStub you can specify:
	- the selector the MethodStub responds to.  This is set when the MethodStub is created.
	- the method arguments the MethodStub responds to.  See the methods in the "arguments" protocol.

Instance Variables:
	argSpec 			<ArgumentSpec> 	Any criteria for the arguments that this MethodStub responds to
	action 				<BlockContext> 		The action to preform whenever this MethodStub is invoked
	invocationCount 		<Integer> 			The number of times that this MethodStub was invoked
	returnValueSpec 	<ReturnValueSpec> 	The value returned when this MethodStub is invoked
	selector 			<Symbol> 			The selector that this MethodStub matches

!

!MethodStub class methodsFor: 'instance creation' stamp: 'dk 17.8.07 11:57'!
newForSelector: aSelector
	^ super new initializeWithSelector: aSelector! !

!MethodStub methodsFor: 'initialize-release' stamp: 'dk 17.8.07 11:57'!
initializeWithSelector: aSelector
	selector := aSelector.
	returnValueSpec := ReturnValueSpec new.
	argSpec := ArgumentSpec new.
	action := nil.
	invocationCount := 0.! !

!MethodStub methodsFor: 'testing' stamp: 'dk 17.8.07 11:57'!
isPromiscuous
	"Does this MethodStub match any/all arguments?"

	^ argSpec isPromiscuous! !

!MethodStub methodsFor: 'testing' stamp: 'dk 17.8.07 11:57'!
isSatisfied
	"Return whether or not this MethodStub is 'satisfied' -- currently, this means that it has been invoked"

	^ invocationCount > 0.! !

!MethodStub methodsFor: 'querying' stamp: 'dk 17.8.07 11:57'!
matches: aMessage
	"Determine whether the selector/arguments criteria of this MethodStub matches the selector and arguments of aMessage."

	| match |
	match := selector = aMessage selector.
	match := match and: [ argSpec matches: aMessage arguments ].

	^ match! !

!MethodStub methodsFor: 'printing' stamp: 'dk 17.8.07 11:57'!
printOn: aStream

	selector printOn: aStream.
	
	argSpec isPromiscuous
		ifTrue: [ aStream nextPutAll: ' with any arguments.' ]
		ifFalse: [
			aStream nextPutAll: ' with arguments: '.
			argSpec printOn: aStream.
		]! !

!MethodStub methodsFor: 'return' stamp: 'dk 17.8.07 11:57'!
returnValueOr: anObject
	"Return this MethodStub's previously set return value, or, if it hasn't been set, return anObject"

	^ returnValueSpec returnValueOr: anObject! !

!MethodStub methodsFor: 'return' stamp: 'dk 17.8.07 11:57'!
returns: anObject
	"Set the value that this MethodStub returns when invoked.  'nil' is a valid return value."

	returnValueSpec returns: anObject.! !

!MethodStub methodsFor: 'actions' stamp: 'dk 17.8.07 11:57'!
runAction
	"Run this MethodStub's action, if defined."

	invocationCount := invocationCount + 1.

	action isNil
		ifFalse: [ action value ]! !

!MethodStub methodsFor: 'querying' stamp: 'dk 17.8.07 11:57'!
selector
	^ selector! !

!MethodStub methodsFor: 'testing' stamp: 'dk 17.8.07 11:57'!
wasInvoked
	"Return whether or not this MethodStub was invoked (ie/ whether or not #runAction was called)"

	^ invocationCount > 0.! !

!MethodStub methodsFor: 'actions' stamp: 'dk 17.8.07 11:57'!
will: aBlock
	"Set the action that's run whenever this MethodStub is invoked."

	action := aBlock! !

!MethodStub methodsFor: 'arguments' stamp: 'dk 17.8.07 11:57'!
with: anObject
	"Add an argument to the method stub.  When the method is called, the arguments that it's called with are compared against to the arguments added to the stub, and if they match then the call is permitted.  If the argument added is a single-argument BlockContext, then it is executed with the called argument and it is expected to return a boolean representing whether there is a match or not.  Otherwise, an equality check is performed.
	Example:
		with: [ :arg | arg > 6 ] -- only match if the passed in argument is > 6
		with: 6 -- only match if the passed in argument = 6
		with: [ :arg | arg = [ aBlockCloser ]] -- only match if the passed in argument = aBlockContext
		with: [ :arg | arg isBlockContext and: [ arg numArgs = 3 ]] -- only match if the passed in argument is a 3-argument BlockContext"

	argSpec add: anObject! !

!MethodStub methodsFor: 'arguments' stamp: 'dk 17.8.07 11:57'!
withAnyArgs
	"Indicate that this MethodStub matches any arguments."

	argSpec withAnyArgs! !

!MethodStub methodsFor: 'arguments' stamp: 'dk 17.8.07 11:57'!
withArgs: aCollection
	"Add a collection of arguments to the MethodStub.  Please see the #with: method for more information regarding possible values and interpretations."

	argSpec addAll: aCollection.! !

Object subclass: #Mock
	instanceVariableNames: 'mockedClass proxy stubs expectations exceptionClass'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SMock-Core'!

!Mock commentStamp: '<historical>' prior: 0!
Mock is the main entry class for the mock objects framework.  A Mock is created using the #newForClass: constructor.  The #newForClass:withException: constructor is provided as an integration point for sMock - SUnit -- it is used to specify what exception class to use instead of MockingException.


Stubs are specified using the #stub: method and expected invocations are set using the #expect: method.  #verify will fail if there are any expectations that haven't been invoked.

Instance Variables:
	mockedClass 	<Class> 		The class that is being mocked
	proxy 			<SMProxy> 		A Proxy for the class being mocked
	stubs 			<Collection> 	The methods that have been stubbed
	expectations 	<Collection> 	The methods that are expected to have been called
	exceptionClass 	<Class> 		The exception class to use for fail-fast exceptions!

!Mock class methodsFor: 'instance creation' stamp: 'dk 17.8.07 11:57'!
new
	"Create a new Mock with no limitations on permitted method calls."

	^ self newForClass: nil withException: nil! !

!Mock class methodsFor: 'instance creation' stamp: 'dk 17.8.07 11:57'!
newForClass: aClass
	"Create a new Mock for the specified Class."

	^ self newForClass: aClass withException: nil! !

!Mock class methodsFor: 'instance creation' stamp: 'dk 17.8.07 11:57'!
newForClass: aClass withException: anExceptionClass
	"Create a new Mock for the specified Class and specify the Exception class to use for signaled exceptions.  IF anExceptionClass is nil, then the default class (MockingException) will be used."

	^ super new initializeWith: aClass andExceptionClass: anExceptionClass! !

!Mock class methodsFor: 'instance creation' stamp: 'dk 17.8.07 11:57'!
newWithException: anExceptionClass
	"Create a new Mock with no limitations on permitted method calls."

	^ self newForClass: nil withException: anExceptionClass! !

!Mock methodsFor: 'accessing' stamp: 'dk 17.8.07 11:57'!
exceptionClass
	^ exceptionClass! !

!Mock methodsFor: 'configuring' stamp: 'dk 17.8.07 11:57'!
expect: aSelector
	"Initialize, and resturn, a MethodStub for the the specified selector.  Expected methods are methods which must be called on the proxy in order for #verify to return true."

	self verifyMockedClassUnderstands: aSelector.

	^ expectations add: (MethodStub newForSelector: aSelector)! !

!Mock methodsFor: 'accessing' stamp: 'dk 17.8.07 11:57'!
expectations
	^ expectations! !

!Mock methodsFor: 'accessing' stamp: 'dk 17.8.07 11:57'!
findMethodStubMatching: aMessage
	"Return the MethodStub which most exactly matches aMessage.  Returns nil if there is no match."

	| stub expectation |

	stub := self findMethodStubMatching: aMessage in: stubs allowingSatisfied: true.
	expectation := self findMethodStubMatching: aMessage in: expectations allowingSatisfied: false.

	"If either stub or expectation are nil, return the other."
	stub ifNil: [ ^ expectation ].
	expectation ifNil: [ ^ stub ].

	"Neither stub nor expectation are nil.  If expectation will match any arguments, then return stub (unless it matches any arguments) -- ie/ give expectation priority."
	stub matchesAnyArguments ifTrue: [ ^ expectation ].
	expectation matchesAnyArguments ifTrue: [ ^ stub ].

	"neither stub nor expectation match *any* arguments, and neither are nil, so return expectation."
	^ expectation! !

!Mock methodsFor: 'accessing-private' stamp: 'dk 17.8.07 11:57'!
findMethodStubMatching: aMessage in: aCollectionOfStubs allowingSatisfied: aBoolean
	"Return the MethodStub in aCollectionOfStubs which most exactly matches aMessage."

	| matchingStubs specificStubs |

	matchingStubs := aCollectionOfStubs select: [ :each | each matches: aMessage ].

	"Reject the MethodStubs that have been 'satisfied'."
	matchingStubs := matchingStubs reject: [ :each | aBoolean not & each isSatisfied ].

	matchingStubs isEmpty ifTrue: [ ^ nil ].

	"'specificStubs' is all the non-promiscuous MethodStubs"
	specificStubs := matchingStubs reject: [ :each | each isPromiscuous ].

	^ specificStubs isEmpty
		ifTrue: [ matchingStubs at: 1 ]
		ifFalse: [ specificStubs at: 1 ].! !

!Mock methodsFor: 'initialize-release' stamp: 'dk 17.8.07 11:57'!
initializeWith: aClass andExceptionClass: anExceptionClass
	mockedClass := aClass.
	exceptionClass := anExceptionClass ifNil: [ MockingException ] ifNotNil: [ anExceptionClass ].
	proxy := nil.
	stubs := OrderedCollection new.
	expectations := OrderedCollection new.! !

!Mock methodsFor: 'accessing' stamp: 'dk 17.8.07 11:57'!
mockedClass
	"Get the class that is being mocked."

	^ mockedClass! !

!Mock methodsFor: 'accessing' stamp: 'dk 17.8.07 11:57'!
proxy
	"Get this Mock's proxy.  Multiple calls to this method will return the same object."

	^ proxy == nil
		ifTrue: [ proxy := SMProxy newForMock: self ]
		ifFalse: [ proxy ]! !

!Mock methodsFor: 'configuring' stamp: 'dk 17.8.07 11:57'!
stub: aSelector 
	"Initialize, and resturn, a MethodStub for the the specified selector.  Stubbed methods are methods which aren't necessarily called on the proxy, but when they *are* called they have a predefined action and/or return value."

	self verifyMockedClassUnderstands: aSelector.

	^stubs add: (MethodStub newForSelector: aSelector)! !

!Mock methodsFor: 'accessing' stamp: 'dk 17.8.07 11:57'!
stubs
	^ stubs! !

!Mock methodsFor: 'verification' stamp: 'dk 8/17/2007 12:56'!
verify
	"Verify that all expectations (those methods indicated with #expect:) have been met (ordering, max requirements and calling unspecified stubs/expectations are all fail-fast).  This will raise a VerificationException for the first expectation which is not met."

	| message |
	message := ''.

	expectations doWithIndex: [ :elem :i |
		elem wasInvoked
			ifFalse: [
				message := message, i printString, '. ', elem printString, (Array with: Character cr).
			]
	].

	message notEmpty
		ifTrue: [
			VerificationException signal: 'Unmet expectations: ', (Array with: Character cr), message.
		]! !

!Mock methodsFor: 'configuring-private' stamp: 'dk 8/17/2007 12:56'!
verifyMockedClassUnderstands: aSelector 
	mockedClass isNil ifTrue: [ ^ self ].

	(mockedClass canUnderstand: aSelector) 
		ifFalse: [
			| message |
			message := mockedClass printString , ' does not understand ' , aSelector printString , '.'.
			exceptionClass signal: message ]! !

!Object methodsFor: '*SMock-Core-testing' stamp: 'dk 17.8.07 11:58'!
isBlockContext
	"Override in BlockContext"

	^ false! !

Object subclass: #ReturnValueSpec
	instanceVariableNames: 'returnValue isSet'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SMock-Core'!

!ReturnValueSpec class methodsFor: 'instance creation' stamp: 'dk 17.8.07 11:57'!
new
	^ super new initialize! !

!ReturnValueSpec methodsFor: 'initialize-release' stamp: 'dk 17.8.07 11:57'!
initialize
	returnValue := BlockOrValue new.
	isSet := false! !

!ReturnValueSpec methodsFor: 'accessing' stamp: 'dk 17.8.07 11:57'!
returnValueOr: alternateValue
	"Return either this ReturnValueSpec's return value (if set) or the alternate value"
	| aValue |
	aValue := BlockOrValue new value: alternateValue.

	^ isSet
		ifFalse: [ aValue value ]
		ifTrue: [ returnValue value ]! !

!ReturnValueSpec methodsFor: 'accessing' stamp: 'dk 17.8.07 11:57'!
returns: aValue
	"Set the return value for this ReturnValueSpec."
	returnValue value: aValue.
	isSet := true! !

Error subclass: #MockingException
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SMock-Core'!

Error subclass: #VerificationException
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SMock-Core'!
SMProxy initialize!
